<?php
/**
 * Get the value of a settings field.
 *
 * @param string $option  settings field name
 * @param string $section the section name this field belongs to
 * @param string $default default text if it's not found
 *
 * @return mixed
 */
function eazydocspro_get_option( $option, $section, $default = '' ) {
	$options = get_option( $section );

	if ( isset( $options[ $option ] ) ) {
		return $options[ $option ];
	}

	return $default;
}

/**
 * Get Page by title
 * @param $title
 * @param $post_type
 *
 * @return int[]|WP_Post[]
 */
function ezdpro_get_page_by_title( $title, $post_type = 'page' ) {
	$posts = get_posts(
		array(
			'post_type'              => $post_type,
			'title'                  => $title,
			'post_status'            => 'all',
			'numberposts'            => 1,
			'update_post_term_cache' => false,
			'update_post_meta_cache' => false,
			'orderby'                => 'post_date ID',
			'order'                  => 'ASC',
		)
	);
	return $posts;
}

/**
 * Get the docs search keywords query
 * @return array|object|stdClass[]|null
 */
function ezd_get_search_keywords() {
    global $wpdb;
	return $wpdb->get_results( "SELECT {$wpdb->prefix}eazydocs_search_keyword.keyword, COUNT(*) AS not_found_count FROM {$wpdb->prefix}eazydocs_search_log JOIN {$wpdb->prefix}eazydocs_search_keyword ON {$wpdb->prefix}eazydocs_search_keyword.id = {$wpdb->prefix}eazydocs_search_log.keyword_id WHERE {$wpdb->prefix}eazydocs_search_log.not_found_count = 1 GROUP BY {$wpdb->prefix}eazydocs_search_keyword.keyword ORDER BY not_found_count DESC LIMIT 20" );
}


if ( ! function_exists( 'eazydocs_breadcrumbs' ) ) {

	/**
	 * Docs breadcrumb.
	 *
	 * @return void
	 */
	function eazydocs_breadcrumbs() {
		global $post;

		$home_text       = eazydocspro_get_option( 'breadcrumb-home-text', 'eazydocs_settings' );
		$front_page      = ! empty ( $home_text ) ? esc_html( $home_text ) : esc_html__( 'Home', 'eazydocs-pro' );
		$docs_home       = eazydocspro_get_option( 'docs-slug', 'eazydocs_settings' );
		$docs_page_title = eazydocspro_get_option( 'docs-page-title', 'eazydocs_settings' );
		$docs_page_title = ! empty ( $docs_page_title ) ? esc_html( $docs_page_title ) : esc_html__( 'Docs', 'eazydocs-pro' );

		$html = '';
		$args = apply_filters( 'eazydocs_breadcrumbs', [
			'delimiter' => '',
			'home'      => $front_page,
			'before'    => '<li class="breadcrumb-item active">',
			'after'     => '</li>',
		] );

		$breadcrumb_position = 1;

		$html .= '<ol class="breadcrumb" itemscope itemtype="http://schema.org/BreadcrumbList">';
		$html .= eazydocs_get_breadcrumb_item( $args['home'], home_url( '/' ), $breadcrumb_position );
		$html .= $args['delimiter'];

		if ( $docs_home ) {
			++ $breadcrumb_position;

			$html .= eazydocs_get_breadcrumb_item( $docs_page_title, get_permalink( $docs_home ), $breadcrumb_position );
			$html .= $args['delimiter'];
		}

		if ( 'docs' == $post->post_type && $post->post_parent ) {
			$parent_id   = $post->post_parent;
			$breadcrumbs = [];

			while ( $parent_id ) {
				++ $breadcrumb_position;

				$page          = get_post( $parent_id );
				$breadcrumbs[] = eazydocs_get_breadcrumb_item( get_the_title( $page->ID ), get_permalink( $page->ID ), $breadcrumb_position );
				$parent_id     = $page->post_parent;
			}

			$breadcrumbs = array_reverse( $breadcrumbs );

			for ( $i = 0; $i < count( $breadcrumbs ); ++ $i ) {
				$html .= $breadcrumbs[ $i ];
				$html .= ' ' . $args['delimiter'] . ' ';
			}
		}

		$html .= ' ' . $args['before'] . get_the_title() . $args['after'];

		$html .= '</ol>';

		echo apply_filters( 'eazydocs_breadcrumbs_html', $html, $args );
	}
}

// Recently Viewed Docs
add_action( 'template_redirect', 'eazydocs_posts_visited' );
function eazydocs_posts_visited() {
	if ( is_single() && get_post_type() == 'docs' ) {
		$cooki    = 'eazydocs_recent_posts';
		$ft_posts = isset( $_COOKIE[ $cooki ] ) ? json_decode( htmlspecialchars( $_COOKIE[ $cooki ], true ) ) : null;
		if ( isset( $ft_posts ) ) {
			// Remove current post in the cookie
			$ft_posts = array_diff( $ft_posts, array( get_the_ID() ) );
			// update cookie with current post
			array_unshift( $ft_posts, get_the_ID() );
		} else {
			$ft_posts = array( get_the_ID() );
		}
		setcookie( $cooki, json_encode( $ft_posts ), time() + ( DAY_IN_SECONDS * 31 ), COOKIEPATH, COOKIE_DOMAIN );
	}
}

/**
 * EazyDocsPro img callback
 *
 * @param $src
 * @param $alt
 */
function eazydocs_pro_img( $src, $alt ) {
	echo "<img src='" . $src . "' alt='" . $alt . "' />";
}

/**
 * EazyDocsPro votes counter
 *
 * @return int
 */
function eazydocs_voted() {

	$args = array(
		'post_type'      => 'docs',
		'posts_per_page' => - 1,
		'post_status'    => array( 'publish' )
	);

	$positive_arr = [];
	$negative_arr = [];

	foreach ( get_posts( $args ) as $posts ) {
		// Positive
		$positive       = get_post_meta( $posts->ID, 'positive_time', true );
		$is_p           = ! empty( $positive ) ? 'p' : '';
		$positive_arr[] = get_post_meta( $posts->ID, 'positive_time', true ) . $is_p;

		// Negative
		$negative       = get_post_meta( $posts->ID, 'negative_time', true );
		$is_n           = ! empty( $negative ) ? 'n' : '';
		$negative_arr[] = get_post_meta( $posts->ID, 'negative_time', true ) . $is_n;

	}
	$positive_arr = array_filter( $positive_arr );
	$negative_arr = array_filter( $negative_arr );
	$vote_dates   = array_merge( $positive_arr, $negative_arr );

	// Sort with vote suffix
	usort( $vote_dates, "main_date_sort" );
	$votes = array_filter( $vote_dates );

	$no_np    = array_merge( $positive_arr, $negative_arr );
	$np_value = [];

	foreach ( $no_np as $pid ) {
		$pid_trim = substr( $pid, - 1 );
		if ( $pid_trim == 'p' ) {
			$vote_text = 'Positive ';
		} elseif ( $pid_trim == 'n' ) {
			$vote_text = 'Negative ';
		}
		$np_value[] = rtrim( $pid, $pid_trim );
	}

	// Sort without vote suffix
	usort( $np_value, "date_sort" );
	$doc_count   = 0;
	$join_arrays = array_combine( $votes, $np_value );
	foreach ( $join_arrays as $np => $item ) :
		$doc_count ++;
		$is_np = substr( $np, - 1 );

		if ( $is_np == 'p' ) {
			$vote_text  = 'Positive ';
			$vote_color = 'green ';
		} elseif ( $is_np == 'n' ) {
			$vote_text  = 'Negative ';
			$vote_color = 'red ';
		}
		$args2      = array(
			'post_type'      => 'docs',
			'posts_per_page' => - 1,
			'meta_query'     => array(
				array(
					'value' => $item
				)
			)
		);
		$date_posts = get_posts( $args2 );
	endforeach;

	return $doc_count;
	wp_reset_postdata();
}

/**
 * EazyDocsPro comments counter
 *
 * @return int
 */
function eazydocs_comment_count() {
	$args     = array(
		'number'      => 5,
		'post_status' => 'publish',
		'post_type'   => array( 'docs' ),
		'parent'      => 0,
	);
	$comments = get_comments( $args );

	return count( $comments );
}

/**
 * Get all WP Roles
 *
 * @return string[]
 */
function eazydocs_user_role_names() {
	global $wp_roles;
	if ( ! isset( $wp_roles ) ) {
		$wp_roles = new WP_Roles();
	}

	return $wp_roles->get_names();
}

/**
 * Limit latter
 *
 * @param        $string
 * @param        $limit_length
 * @param string $suffix
 */
function assistant_limit_letter( $string, $limit_length, $suffix = '...' ) {
	if ( strlen( $string ) > $limit_length ) {
		echo strip_shortcodes( substr( $string, 0, $limit_length ) . $suffix );
	} else {
		echo strip_shortcodes( esc_html( $string ) );
	}
}

/**
 * Get total views
 *
 * @return int
 */
function ezdpro_get_total_views() {
	global $wpdb;
	// get total views count from wp_eazy_docs_view table and check if empty then set 0
	$total_views = $wpdb->get_var( "SELECT SUM(count) FROM {$wpdb->prefix}eazydocs_view_log" );

	return $total_views + $wpdb->get_var( "SELECT SUM(meta_value) FROM {$wpdb->prefix}postmeta WHERE meta_key = 'post_views_count'" );
}

//CUSTOM META BOX
add_action( 'add_meta_boxes', function () {
	add_meta_box( 'EZD Feedback Options', 'EZD Feedback Options', 'ezd_feedback_docs', 'ezd_feedback' );
	add_meta_box( 'doc_extra_information', 'Additional Information', 'ezd_docs_extra_info', 'docs' );
} );

function ezd_feedback_docs() {
	?>
    <p><b>ID</b><br/>
        <input type="text" name="ezd_feedback_id" value="<?php echo get_post_meta( get_the_ID(), 'ezd_feedback_id', true ); ?>" class="widefat"/>
    </p>
    <p><b>Name</b><br/>
        <input type="text" name="ezd_feedback_name" value="<?php echo get_post_meta( get_the_ID(), 'ezd_feedback_name', true ); ?>" class="widefat"/>
    </p>
    <p><b>Email</b><br/>
        <input name="ezd_feedback_email" value="<?php echo get_post_meta( get_the_ID(), 'ezd_feedback_email', true ); ?>" class="widefat"/>
    </p>
    <p><b>Subject</b><br/>
        <input name="ezd_feedback_subject" value="<?php echo get_post_meta( get_the_ID(), 'ezd_feedback_subject', true ); ?>" class="widefat"/>
    </p>
    <p><b>Status</b><br/>
        <input name="ezd_feedback_status" value="<?php echo get_post_meta( get_the_ID(), 'ezd_feedback_status', true ); ?>" class="widefat"/>
    </p>
<?php }

function ezd_docs_extra_info() {
	?>
    <p><b><?php echo esc_html( 'Doc Contributors' ); ?></b><br/>
        <input type="text" name="ezd_doc_contributors" value="<?php echo get_post_meta( get_the_ID(), 'ezd_doc_contributors', true ); ?>" class="widefat"/>
    </p>
    <p><b><?php echo esc_html( 'Read / Unread' ); ?></b><br/>
        <input type="text" name="ezd_doc_read_unread" value="<?php echo get_post_meta( get_the_ID(), 'ezd_doc_read_unread', true ); ?>" class="widefat"/>
    </p>
    <p><b><?php echo esc_html( 'Read / Unread Time' ); ?></b><br/>
        <input type="text" name="ezd_doc_read_unread_time" value="<?php echo get_post_meta( get_the_ID(), 'ezd_doc_read_unread_time', true ); ?>"
               class="widefat"/>
    </p>
    <p><b><?php echo esc_html( 'Left Sidebar' ); ?></b><br/>
        <input type="text" name="ezd_doc_left_sidebar_type" value="<?php echo get_post_meta( get_the_ID(), 'ezd_doc_left_sidebar_type', true ); ?>"
               class="widefat"/><br/>
        <textarea name="ezd_doc_left_sidebar" id="" rows="5" class="widefat"> <?php echo get_post_meta( get_the_ID(), 'ezd_doc_left_sidebar',
				true ); ?> </textarea>
    </p>
    <p><b><?php echo esc_html( 'Right Sidebar' ); ?></b><br/>
        <input type="text" name="ezd_doc_right_sidebar_type" value="<?php echo get_post_meta( get_the_ID(), 'ezd_doc_right_sidebar_type', true ); ?>"
               class="widefat"/><br/>
        <textarea name="ezd_doc_right_sidebar" id="" rows="5" class="widefat"> <?php echo get_post_meta( get_the_ID(), 'ezd_doc_right_sidebar',
				true ); ?> </textarea>
    </p>
<?php }

add_action( 'save_post', function ( $post_id ) {
	// Feedback Options
	$ezd_feedback_id          = $_POST['ezd_feedback_id'] ?? '';
	$ezd_feedback_name        = $_POST['ezd_feedback_name'] ?? '';
	$ezd_feedback_email       = $_POST['ezd_feedback_email'] ?? '';
	$ezd_feedback_subject     = $_POST['ezd_feedback_subject'] ?? '';
	$ezd_feedback_status      = $_POST['ezd_feedback_status'] ?? '';
	$ezd_doc_contributors     = $_POST['ezd_doc_contributors'] ?? '';
	$ezd_doc_read_unread      = $_POST['ezd_doc_read_unread'] ?? '';
	$ezd_doc_read_unread_time = $_POST['ezd_doc_read_unread_time'] ?? '';

	$ezd_doc_left_sidebar  = $_POST['ezd_doc_left_sidebar'] ?? '';
	$ezd_doc_right_sidebar = $_POST['ezd_doc_right_sidebar'] ?? '';

	update_post_meta( $post_id, 'ezd_feedback_id', $ezd_feedback_id );
	update_post_meta( $post_id, 'ezd_feedback_name', $ezd_feedback_name );
	update_post_meta( $post_id, 'ezd_feedback_email', $ezd_feedback_email );
	update_post_meta( $post_id, 'ezd_feedback_subject', $ezd_feedback_subject );
	update_post_meta( $post_id, 'ezd_feedback_status', $ezd_feedback_status );
	update_post_meta( $post_id, 'ezd_doc_read_unread', $ezd_doc_read_unread );
	update_post_meta( $post_id, 'ezd_doc_read_unread_time', $ezd_doc_read_unread_time );

	update_post_meta( $post_id, 'ezd_doc_left_sidebar', $ezd_doc_left_sidebar );
	update_post_meta( $post_id, 'ezd_doc_right_sidebar', $ezd_doc_right_sidebar );

	$ezd_doc_revisions     = wp_get_post_revisions( get_the_ID() );
	$ezd_doc_revision_list = '';
	foreach ( $ezd_doc_revisions as $ezd_doc_revision ) {
		$ezd_doc_revision_list .= $ezd_doc_revision->post_author . ',';
	}

	update_post_meta( $post_id, 'ezd_doc_contributors', $ezd_doc_revision_list );
} );

// User Feedback archive menu active
add_action( 'admin_footer', function () { ?>
    <script>
        // User feedback archived
        feedback_archived = "admin.php?page=ezd-user-feedback-archived";
        doc_badge = 'edit-tags.php?taxonomy=doc_badge&post_type=docs';

        // EazyDocs menu active when it's EazyDocs screen
        if (window.location.href.indexOf(feedback_archived) > -1) {
            jQuery('.toplevel_page_eazydocs').removeClass('wp-not-current-submenu').addClass('wp-has-current-submenu wp-menu-open').find('li').has('a[href*="admin.php?page=ezd-user-feedback"]').addClass('current');
        }
        if (window.location.href.indexOf(doc_badge) > -1) {
            jQuery('.toplevel_page_eazydocs').removeClass('wp-not-current-submenu').addClass('wp-has-current-submenu wp-menu-open').find('li').has('a[href*="edit-tags.php?taxonomy=doc_badge&post_type=docs"]').addClass('current');
        }
    </script>
<?php } );

/**
 * Get post by slug
 * @param string $post_slug
 * @return string
 */
function get_post_by_slug($post_slug) {
    global $wpdb;
    $post_slug 	= '%' . $post_slug . '%';
	$post 		= '';

	// if table exists
	if ( $wpdb->get_var("SHOW TABLES LIKE 'wp_term_relationships'") == 'wp_term_relationships' ) {
		$post = $wpdb->get_var($wpdb->prepare("SELECT ID FROM {$wpdb->posts} LEFT JOIN {$wpdb->term_relationships} ON {$wpdb->posts}.ID = {$wpdb->term_relationships}.object_id WHERE post_name LIKE %s ", $post_slug));
	}
    return $post;
}

/**
 * Private Doc visibility
 * Guest Login / Login Required
 * Login with custom form
**/
function ezd_private_login() {
	if ( is_404() ) {		
		global $wp_query;

		global $wpdb;
        $query      = $wp_query->request;
        $statuses   = $wpdb->get_results($query);
        
        if ($statuses) {
            foreach ($statuses as $status) {
                $is_post_status = $status->post_status;
                $is_post_id 	= $status->ID;
            }
        }
		
		if ( $is_post_status == 'private' && get_post_type($is_post_id) == 'docs' ) {
			$doc_user_mode 			= ezd_get_opt('private_doc_mode');
			$add_user_page_id 		= ezd_get_opt('private_doc_login_page');
			$add_user_page_id       = get_permalink($add_user_page_id);

			if ( $doc_user_mode == 'login' && ! empty( $add_user_page_id ) ) {		
				if ( ! is_user_logged_in() ) {
					global $wp;
					$redirect_to            = home_url( $wp->request );
					wp_safe_redirect( $add_user_page_id.'?after_login='.$redirect_to.'&private_doc=yes' );
				}
			}
		}
	}
}

// Run before the headers and cookies are sent.
add_action( 'wp', 'ezd_private_login' );

/**
 * Doc contributor login
 * @return void
 */
/**
 * Doc contributor login
 * @return void
 */
function doc_contributor_login(){
	if ( ! empty( eaz_fs()->is_plan('promax') ) ) {

		$doc_contribution 		= '';
		if( function_exists('ezd_get_opt') ){
			$doc_contribution 	= ezd_get_opt('is_doc_contribution');
		}

		if( $doc_contribution ){
			if( isset( $_GET['ezd_login'] ) ) {
				$ezd_username 	= $_GET['ezd_username'] ?? '';
				$ezd_password 	= $_GET['ezd_password'] ?? '';

				if( ! empty( $ezd_username ) && ! empty( $ezd_password ) ){
					$creds = array(
						'user_login'    => $ezd_username,
						'user_password' => $ezd_password,
						'remember'      => true
					);
					wp_signon( $creds, true );

					if( !empty($_GET['add_new_doc']) ){
						if( wp_safe_redirect( $_GET['ezd_private_doc'] ) ) {
							die;
						}
					}

					if( ! empty( $_GET['ezd_private_doc'])  && empty( $_GET['private_doc'] ) ){
						if( wp_safe_redirect( admin_url('post.php?post='. $_GET['ezd_private_doc'] . '&action=edit') ) ) {
							die;
						}
					}

				}
			}
		}
	}
}
add_action('after_setup_theme', 'doc_contributor_login');

/**
 * EazyDocs Login Form
 * @param array $atts
 * @param array $content
 * @return void
 */
add_shortcode( 'ezd_login_form', function ( $atts, $content = [] ) {
	$ezd_login_atts = shortcode_atts( [
		'login_title'      => __( 'You must log in to continue.', 'eazydocs-pro' ),
		'login_subtitle'   => __( 'Login to ' . get_bloginfo(), 'eazydocs-pro' ),
		'login_btn'        => __( 'Log In', 'eazydocs-pro' ),
		'login_forgot_btn' => __( 'Forgotten account?', 'eazydocs-pro' ),

	], $atts );

	ob_start();
	if ( is_user_logged_in() ) {
		/* Silence is golden */
	} else {
		eazydocs_get_template( 'ezd-login.php', [
			'login_title'      => $ezd_login_atts['login_title'] ?? '',
			'login_subtitle'   => $ezd_login_atts['login_subtitle'] ?? '',
			'login_btn'        => $ezd_login_atts['login_btn'] ?? '',
			'login_forgot_btn' => $ezd_login_atts['login_forgot_btn'] ?? '',
		] );
	}

	return ob_get_clean();
} );

add_action( 'wp', function () {
	$ezd_login_page_id     = '';
	$ezd_login_private_doc = '';
	if ( function_exists( 'ezd_get_opt' ) ) {
		$ezd_login_page_id     = ezd_get_opt( 'docs_frontend_login_page' );
		$ezd_login_private_doc = ezd_get_opt( 'private_doc_login_page' );
	}

	global $wp_query;
	$slug           = $_GET['ezd_private_doc'] ?? '';
	$is_post_status = get_post_status( url_to_postid( $slug ) );

	if ( ! empty( $ezd_login_page_id ) ) {
		if ( is_page( $ezd_login_page_id ) ) {
			if ( is_user_logged_in() ) {
				if ( $is_post_status != 'private' ) {
					wp_safe_redirect( home_url() );
				} else {
					wp_safe_redirect( $slug );
				}
			} else {
				if ( isset( $_GET['ezd_login'] ) ) {
					$ezd_username = $_GET['ezd_username'] ?? '';
					$ezd_password = $_GET['ezd_password'] ?? '';

					if ( ! empty( $ezd_username ) && ! empty( $ezd_password ) ) {

						$creds = array(
							'user_login'    => $ezd_username,
							'user_password' => $ezd_password,
							'remember'      => true
						);
						wp_signon( $creds, true );

						if ( ! empty( $_GET['private_doc'] ) ) {
							wp_safe_redirect( $_GET['ezd_private_doc'] );
						}

					}
				}
			}
		}
	}

	if ( ! empty( $ezd_login_private_doc ) ) {
		if ( is_page( $ezd_login_private_doc ) ) {
			if ( is_user_logged_in() ) {
				global $wp;
				$redirect_tos = $_GET['after_login'] ?? '';
				wp_safe_redirect( $redirect_tos );
			} else {
				if ( isset( $_GET['ezd_login'] ) ) {
					$ezd_username = $_GET['ezd_username'] ?? '';
					$ezd_password = $_GET['ezd_password'] ?? '';

					if ( ! empty( $ezd_username ) && ! empty( $ezd_password ) ) {
						$creds = array(
							'user_login'    => $ezd_username,
							'user_password' => $ezd_password,
							'remember'      => true
						);
						wp_signon( $creds, true );

						if ( ! empty( $_GET['private_doc'] ) ) {
							wp_safe_redirect( $_GET['ezd_private_doc'] );
						}

					}
				}
			}
		}
	}

	if ( ! empty( $_GET['edit_doc_url'] ) ) {
		if ( ! is_user_logged_in() ) {
			$add_user_page_id = ezd_get_opt( 'docs_frontend_login_page' );
			$add_user_page_id = get_permalink( $add_user_page_id );

			global $wp;
			$redirect_to = $_GET['edit_doc_url'] ?? '';
			wp_safe_redirect( $add_user_page_id . '?after_login=' . $redirect_to );
			eazydocs_get_template_part( 'ezd-login' );
			die;
		}
	}

	if ( ! empty( $_GET['add_new'] ) ) {
		if ( ! is_user_logged_in() ) {
			$add_user_page_id = ezd_get_opt( 'docs_frontend_login_page' );
			$add_user_page_id = get_permalink( $add_user_page_id );

			global $wp;
			$redirect_to = $_GET['add_new'] ?? '';
			wp_safe_redirect( $add_user_page_id . '?after_login=' . $redirect_to . '&add_new_doc=yes' );
			eazydocs_get_template_part( 'ezd-login' );
			die;
		}
	}

} );

add_action( 'wp_head', function () {
	$doc_login    = ezdpro_get_page_by_title( 'Documentation Login' );
	$doc_login_id = $doc_login->ID ?? '';
	?>
    <style>
        <?php echo '.page-id-'.$doc_login_id; ?>
        .ezd_doc_login_form {
            margin: auto;
            width: 515px;
        }

        <?php echo '.page-id-'.$doc_login_id; ?>
        .ezd_doc_login_wrap {
            background-color: #e9ebee;
            width: 100%;
            height: 100vh;
            display: flex;
        }

        <?php echo '.page-id-'.$doc_login_id; ?>
        .ezd_doc_login_form input {
            width: 300px
        }

        <?php echo '.page-id-'.$doc_login_id; ?>
        .ezd-login-form-wrap {
            padding: 22px 108px 26px;
        }
    </style>
<?php } );


add_action( 'wp_ajax_ezd_doc_contributor', 'ezd_doc_contributor' );
add_action( 'wp_ajax_nopriv_ezd_doc_contributor', 'ezd_doc_contributor' );
function ezd_doc_contributor( $doc_id_int ) {

	$is_doc_delete = $_POST['contributor_delete'] ?? '';
	$is_doc_add    = $_POST['contributor_add'] ?? '';

	if ( $is_doc_delete ) {

		$ezd_doc_contributor_list = get_post_meta( $_POST['data_doc_id'], 'ezd_doc_contributors', true );
		$ezd_doc_contributor_list = explode( ',', $ezd_doc_contributor_list );

		$is_doc_delete_unique = array_diff( $ezd_doc_contributor_list, [ $is_doc_delete ] );

		$is_doc_delete_ids = '';
		foreach ( $is_doc_delete_unique as $is_doc_delete_id ) {
			$is_doc_delete_ids .= $is_doc_delete_id . ',';
		}
		update_post_meta( $_POST['data_doc_id'], 'ezd_doc_contributors', $is_doc_delete_ids );
		?>

        <ul class="users_wrap_item <?php echo esc_attr( 'to-add-user-' . $is_doc_delete ); ?>" id="<?php echo esc_attr( 'to-add-user-' . $is_doc_delete ); ?>">
            <li>
                <a href='<?php echo get_author_posts_url( $is_doc_delete ); ?>'>
					<?php echo get_avatar( $is_doc_delete, '35' ); ?>
                </a>
            </li>
            <li>
                <a href='<?php echo get_author_posts_url( $is_doc_delete ); ?>'>
					<?php echo get_the_author_meta( 'display_name', $is_doc_delete ); ?>
                </a>
                <span>
					<?php echo get_the_author_meta( 'user_email', $is_doc_delete ); ?>
				</span>
            </li>
            <li>
                <a data_name="<?php echo get_the_author_meta( 'display_name', $is_doc_delete ); ?>" class="ezd_contribute_add circle-btn" data-contributor-add="<?php echo esc_attr( $is_doc_delete ); ?>"
                   data-doc-id="<?php echo esc_attr( $_POST['data_doc_id'] ); ?>"> &plus; </a>
            </li>
        </ul>
		<?php
	}

	if ( $is_doc_add ) {
		$ezd_doc_contributor_list = get_post_meta( $_POST['data_doc_id'], 'ezd_doc_contributors', true );
		$ezd_doc_contributors     = $ezd_doc_contributor_list . $is_doc_add . ',';
		update_post_meta( $_POST['data_doc_id'], 'ezd_doc_contributors', $ezd_doc_contributors );
		?>
        <ul class="users_wrap_item <?php echo esc_attr( 'user-' . $is_doc_add ); ?>" id="<?php echo esc_attr( 'user-' . $is_doc_add ); ?>">
            <li>
                <a href='<?php echo get_author_posts_url( $is_doc_add ); ?>'>
					<?php echo get_avatar( $is_doc_add, '35' ); ?>
                </a>
            </li>
            <li>
                <a href='<?php echo get_author_posts_url( $is_doc_add ); ?>'>
					<?php echo get_the_author_meta( 'display_name', $is_doc_add ); ?>
                </a>
                <span>
					<?php echo get_the_author_meta( 'user_email', $is_doc_add ); ?>
				</span>
            </li>
            <li>
                <a data_name="<?php echo get_the_author_meta( 'display_name', $is_doc_add ); ?>"  class="ezd_contribute_delete circle-btn" data-contributor-delete="<?php echo esc_attr( $is_doc_add ); ?>"
                   data-doc-id="<?php echo esc_attr( $_POST['data_doc_id'] ); ?>">
                    &times;
                </a>
            </li>
        </ul>
		<?php
	}
	wp_die();
}

/**
 * Admin assets
 *
 * @return bool|void
 */
function ezydocspro_admin_assets() {
	$admin_page = $_GET['page'] ?? '';
	$post_type  = $_GET['post_type'] ?? '';

	if ( strstr( $_SERVER['REQUEST_URI'], 'wp-admin/users.php' ) || $admin_page == 'eazydocs' || $admin_page == 'eazydocs-settings'
	     || $admin_page == 'ezd-analytics'
	     || $admin_page == 'ezd-user-feedback'
	     || $admin_page == 'ezd-user-feedback-archived'
	     || $post_type == 'onepage-docs'
	) {
		return true;
	}
}

/**
 * Number format function
 *
 * @param $number
 *
 * @return mixed|string
 */
function eazydocspro_number_format( $number ) {
	if ( $number >= 1000 && $number < 1000000 ) {
		$number = round( $number / 1000, 1 ) . 'k';
	} elseif ( $number >= 1000000 && $number < 1000000000 ) {
		$number = round( $number / 1000000, 1 ) . 'm';
	} elseif ( $number >= 1000000000 && $number < 1000000000000 ) {
		$number = round( $number / 1000000000, 1 ) . 'b';
	} elseif ( $number >= 1000000000000 ) {
		$number = round( $number / 1000000000000, 1 ) . 't';
	}

	return $number;
}

/**
 * Frontend assets
 *
 * @return bool|void
 */
function ezydocspro_frontend_assets() {
	global $post;
	$post_content_check = $post->post_content ?? '';

	if ( in_array( 'eazydocs_shortcode', get_body_class() ) || has_shortcode( $post_content_check, 'ezd_login_form' ) || is_singular( 'docs' )
	     || is_singular( 'onepage-docs' )
	) {
		return true;
	}
}

/**
 * Assistant assets
 *
 * @return bool|void
 */
function eazydocspro_assistant_assets() {
	$opt       = get_option( 'eazydocs_settings' );
	$assistant = $opt['assistant_visibility'] ?? '1';

	if ( $assistant == 1 ) {
		return true;
	}
}

// Get top level parent doc id
function get_root_parent_id( $page_id ) {
	global $wpdb;
	$parent = $wpdb->get_var( "SELECT post_parent FROM $wpdb->posts WHERE post_type='docs' AND post_status='publish' AND ID = '$page_id'" );
	if ( $parent == 0 ) {
		return $page_id;
	} else {
		return get_root_parent_id( $parent );
	}
}

/**
 * Assign parent to new doc
 *
 * @param string  $post_content
 * @param WP_Post $post
 *
 * @return string
 */
function assign_parent_to_new_doc( $post_content, $post ) {
	if ( $post->post_type != 'docs' ) {
		return $post_content;
	}
	$post->post_parent = $_GET['ezd_doc_parent'] ?? ''; //Parent post_id goes here
	wp_update_post( $post );

	return $post_content;
}

add_filter( 'default_content', 'assign_parent_to_new_doc', 10, 2 );

/**
 * Count total helpful docs from post meta positive and negative
 */
function count_total_helpful_docs() {
	// count all positive and negative from wp-postmeta
	global $wpdb;
	$prefix = $wpdb->prefix;
	$positive
		= $wpdb->get_var( "SELECT COUNT(*) FROM {$prefix}postmeta WHERE meta_key = 'positive'" );
	$negative
		= $wpdb->get_var( "SELECT COUNT(*) FROM {$prefix}postmeta WHERE meta_key = 'negative'" );

	$total = $positive + $negative;

	return $total;
}


// Overview Chart filer by date range
function edz_filter_overview_date() {
	global $wpdb;
	$start_date = sanitize_text_field( $_POST['startDate'] );
	$end_date   = sanitize_text_field( $_POST['endDate'] );

	$posts = $wpdb->get_results( "SELECT post_id, SUM(count) AS totalcount, created_at FROM {$wpdb->prefix}eazydocs_view_log WHERE created_at BETWEEN '$start_date' AND '$end_date' GROUP BY post_id" );

	// get data from wp_eazydocs_search_log base on $start_date and $end_date with prefix
	$prefix              = $wpdb->prefix;
	$eazydocs_search_log = $prefix . 'eazydocs_search_log';
	$eazydocs_search_log_data
	                     = $wpdb->get_results( "SELECT * FROM $eazydocs_search_log WHERE created_at BETWEEN '$start_date' AND '$end_date' GROUP BY created_at" );

	$labels              = [];
	$ViewsCount          = [];
	$Liked               = [];
	$Disliked            = [];
	$searchCount         = [];
	$searchCountNotFound = [];

	$m  = date( "m" ); //current month
	$de = date( "d" ); //current day
	$y  = date( "Y" ); //current year

	// Generating apexChart labels
	$datediff = strtotime( $end_date ) - strtotime( $start_date );
	$datediff = floor( $datediff / ( 60 * 60 * 24 ) );

	for ( $i = 0; $i < $datediff + 1; $i ++ ) {
		$labels[]              = date( "d M, Y", strtotime( $start_date . ' + ' . $i . 'day' ) );
		$ViewsCount[]          = 0;
		$Liked[]               = 0;
		$Disliked[]            = 0;
		$searchCount[]         = 0;
		$searchCountNotFound[] = 0;
	}

	// Get 7 data date wise
	foreach ( $posts as $key => $item ) {
		$dates = date( 'd M, Y', strtotime( $item->created_at ) );

		foreach ( $labels as $datekey => $weekdays ) {
			if ( $weekdays == $dates ) {
				$Liked[ $datekey ]    = $Liked[ $datekey ] + (int) get_post_meta( $item->post_id, 'positive', false );
				$Disliked[ $datekey ] = $Disliked[ $datekey ] + (int) get_post_meta( $item->post_id, 'negative', false );
				if ( get_post_meta( $item->post_id, 'post_views_count', true ) ) {
					$ViewsCount[ $datekey ] = get_post_meta( $item->post_id, 'post_views_count', true ) + $item->totalcount;
				} else {
					$ViewsCount[ $datekey ] = $item->totalcount;
				}
			}

			// if $weekdays equal to array data $eazydocs_search_log_data created_at then add count and not_found_count
			if ( $weekdays == date( 'd M, Y', strtotime( $eazydocs_search_log_data[ $datekey ]->created_at ) ) ) {
				$searchCount[ $datekey ] = $searchCount[ $datekey ] + $eazydocs_search_log_data[ $datekey ]->count;
				$searchCountNotFound[ $datekey ] = $searchCountNotFound[ $datekey ] + $eazydocs_search_log_data[ $datekey ]->not_found_count;
			}
		}
	}

	wp_send_json_success( array(
		'labels'              => $labels,
		'views'               => $ViewsCount,
		'liked'               => $Liked,
		'disliked'            => $Disliked,
		'searchCount'         => $searchCount,
		'searchCountNotFound' => $searchCountNotFound
	) );
	wp_die();  //die();
}

add_action( 'wp_ajax_edz_filter_overview_date', 'edz_filter_overview_date' );

// View Chart action by date range
function view_date_range_data() {
	global $wpdb;

	$start_date = $_POST['start_date'];
	$end_date   = $_POST['end_date'];

	$posts
		= $wpdb->get_results( "SELECT post_id, SUM(count) AS totalcount, created_at FROM {$wpdb->prefix}eazydocs_view_log WHERE created_at BETWEEN '$start_date' AND '$end_date' GROUP BY post_id" );

	$labels     = [];
	$ViewsCount = [];

	$m  = date( "m" ); //current month
	$de = date( "d" ); //current day
	$y  = date( "Y" ); //current year

	// Generating apexChart labels
	$datediff = strtotime( $end_date ) - strtotime( $start_date );
	$datediff = floor( $datediff / ( 60 * 60 * 24 ) );

	for ( $i = 0; $i < $datediff + 1; $i ++ ) {
		$labels[]     = date( "d M, Y", strtotime( $start_date . ' + ' . $i . 'day' ) );
		$ViewsCount[] = 0;
	}


	foreach ( $posts as $key => $item ) {
		$dates = date( 'd M, Y', strtotime( $item->created_at ) );

		foreach ( $labels as $datekey => $weekdays ) {
			if ( $weekdays == $dates ) {
				if ( get_post_meta( $item->post_id, 'post_views_count', true ) ) {
					$ViewsCount[ $datekey ] = get_post_meta( $item->post_id, 'post_views_count', true ) + $item->totalcount;
				} else {
					$ViewsCount[ $datekey ] = $item->totalcount;
				}
			}
		}

	}


	wp_reset_postdata();

	wp_send_json_success( array( 'labels' => $labels, 'views' => $ViewsCount ) );
	wp_die();  //die();
}

// add action
add_action( 'wp_ajax_view_date_range_data', 'view_date_range_data' );


// Feedback Chart action by date range
function edz_filter_date_from_feedback() {
	global $wpdb;

	$start_date = sanitize_text_field( $_POST['startDate'] );
	$end_date   = sanitize_text_field( $_POST['endDate'] );

	$start_date = date( 'Y-m-d', strtotime( $start_date ) );
	$end_date   = date( 'Y-m-d', strtotime( $end_date ) );

	// get wp_postmeta base on $start_date and $end_date and post_type = docs and meta_key = positive_time and negative_time where between $start_date and $end_date
	$posts
		= $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}postmeta WHERE post_id IN (SELECT ID FROM {$wpdb->prefix}posts WHERE post_type = 'docs') AND meta_key IN ('positive_time', 'negative_time') AND meta_value BETWEEN '$start_date' AND '$end_date' ORDER BY meta_value DESC" );

	$labels   = [];
	$Liked    = [];
	$Disliked = [];

	$m  = date( "m" ); //current month
	$de = date( "d" ); //current day
	$y  = date( "Y" ); //current year

	// Generating apexChart labels
	$datediff = strtotime( $end_date ) - strtotime( $start_date );
	$datediff = floor( $datediff / ( 60 * 60 * 24 ) );

	for ( $i = 0; $i < $datediff + 1; $i ++ ) {
		$labels[]   = date( "d M, Y", strtotime( $start_date . ' + ' . $i . 'day' ) );
		$Liked[]    = 0;
		$Disliked[] = 0;
	}

	// Get 7 data date wise
	foreach ( $posts as $key => $item ) {
		$dates = date( 'd M, Y', strtotime( $item->meta_value ) );

		foreach ( $labels as $datekey => $weekdays ) {
			# code...
			if ( $weekdays == date( 'd M, Y', strtotime( $item->meta_value ) ) ) {
				// sum of positive_time and negative_time
				if ( $item->meta_key == 'positive_time' ) {
					$Liked[ $datekey ] = $Liked[ $datekey ] + 1;
				} else {
					$Disliked[ $datekey ] = $Disliked[ $datekey ] + 1;
				}

			}
		}
	}

	wp_send_json_success( array( 'labels' => $labels, 'liked' => $Liked, 'disliked' => $Disliked ) );
	wp_die();  //die();
}

// add action
add_action( 'wp_ajax_edz_filter_date_from_feedback', 'edz_filter_date_from_feedback' );


// Search Chart action by date range
function edz_filter_date_from_search() {
	global $wpdb;

	$start_date = sanitize_text_field( $_POST['startDate'] );
	$end_date   = sanitize_text_field( $_POST['endDate'] );

	$search_keyword
		= $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}eazydocs_search_log WHERE created_at BETWEEN '$start_date' AND '$end_date' ORDER BY created_at DESC" );

	$labels              = [];
	$total_search        = [];
	$searchCount         = [];
	$searchCountNotFound = [];

	$m  = date( "m" ); //current month
	$de = date( "d" ); //current day
	$y  = date( "Y" ); //current year

	// Generating apexChart labels
	$datediff = strtotime( $end_date ) - strtotime( $start_date );
	$datediff = floor( $datediff / ( 60 * 60 * 24 ) );

	for ( $i = 0; $i < $datediff + 1; $i ++ ) {
		$labels[]              = date( "d M, Y", strtotime( $start_date . ' + ' . $i . 'day' ) );
		$total_search[]        = 0;
		$searchCount[]         = 0;
		$searchCountNotFound[] = 0;
	}


	// Get 7 data date wise
	foreach ( $search_keyword as $key => $item ) {

		foreach ( $labels as $datekey => $weekdays ) {
			if ( $weekdays == date( 'd M, Y', strtotime( $item->created_at ) ) ) {
				$total_search[ $datekey ]        = count( $search_keyword );
				$searchCount[ $datekey ]         = array_sum( array_column( $search_keyword, 'count' ) );
				$searchCountNotFound[ $datekey ] = array_sum( array_column( $search_keyword, 'not_found_count' ) );
			}
		}

	}

	wp_send_json_success( array(
		'labels'              => $labels,
		'searchCount'         => $searchCount,
		'totalSearch'         => $total_search,
		'searchCountNotFound' => $searchCountNotFound
	) );
	wp_die();  //die();
}

// add action
add_action( 'wp_ajax_edz_filter_date_from_search', 'edz_filter_date_from_search' );


/**
 * Search Helpful Docs
 */
add_action('wp_ajax_search_helpful_docs_paginate', 'search_helpful_docs_paginate');
add_action('wp_ajax_nopriv_search_helpful_docs_paginate', 'search_helpful_docs_paginate');

function search_helpful_docs_paginate() {
	$type = isset($_GET['type']) ? $_GET['type'] : 'most_helpful';

	if( $type == 'most_helpful') {
		if (isset($_GET['total_page'])) {
			$total_page = intval($_GET['total_page']);

			$posts     = get_posts( [ 'post_type' => 'docs', 'posts_per_page'   => -1, 'inclusive' => true] );

			$post_data = [];
			foreach ( $posts as $key => $post ) {
				$post_data[$key]['post_id'] = $post->ID;
				$post_data[$key]['post_title'] = $post->post_title;
				$post_data[$key]['post_edit_link'] = get_edit_post_link( $post->ID );
				$post_data[$key]['post_permalink'] = get_permalink( $post->ID );
				// sum of total positive votes for a post
				$post_data[$key]['positive_time'] = array_sum( get_post_meta( $post->ID, 'positive', false ) );

				$post_data[$key]['negative_time'] = array_sum( get_post_meta( $post->ID, 'negative', false ) );
			}
			// if post has positive_time number large then negative_time number then show large number first
			usort( $post_data, function( $a, $b ) {
				return $b['positive_time'] <=> $a['positive_time'];
			} );

			foreach ( $post_data as $key => $post ) {
				if ( $post['positive_time'] > 0 ) {
					if ( $key >= 0 && $key <= $total_page ) {
						?>
						<li class="dd-item dd3-item dd-item-parent easydocs-accordion-item  ez-section-acc-item type-docs" data-id="<?php echo $post['post_id']; ?>">
						<div class="dd3-content">
							<div class="accordion-title ez-section-title expand--child has-child">
								<div class="left-content">
									<?php
									$edit_link = 'javascript:void(0)';
									$target = '_self';
									if (current_user_can('editor') || current_user_can('administrator')) {
										$edit_link = $post['post_edit_link'];
										$target = '_blank';
									}
									?>
									<h4>
										<a href="<?php echo esc_attr($edit_link); ?>" target="<?php echo esc_attr($target); ?>">
											<?php echo $post['post_title']; ?>
										</a>
									</h4>
									<ul class="actions">
										<li>
											<a href="<?php echo $post['post_permalink']; ?>" target="_blank" title="<?php esc_attr_e('View this doc item in new tab', 'eazydocs-pro') ?>">
											<span class="dashicons dashicons-external"></span>
											</a>
										</li>
									</ul>
								</div>
								<div class="right-content">
									<?php
									$positive = $post['positive_time'];
									$negative = $post['negative_time'];
		
									$positive_title      = $positive ? sprintf(_n('%d Positive vote, ', '%d Positive votes and ', $positive, 'eazydocs-pro' ), number_format_i18n($positive)) : esc_html__('No Positive votes, ', 'eazydocs-pro' );
									$negative_title      = $negative ? sprintf(_n('%d Negative vote found.', '%d Negative votes found.', $negative, 'eazydocs-pro' ), number_format_i18n($negative)) : esc_html__('No Negative votes.', 'eazydocs-pro' );
		
									$positive_icon = '';
									if ( $positive || $negative ) {
										$positive_icon = '<div class="votes"> <div class="like"> <span class="t-success dashicons dashicons-thumbs-up"></span> '.$positive.'</div>';
										$positive_icon .= $negative > 0 ? '<div class="dislike"> <span class="t-danger dashicons dashicons-thumbs-down"></span> '.$negative.'</div>' : '';
										$positive_icon .= '</div>';
									}
		
									$sum_votes = $positive + $negative;
									echo $positive_icon;
									?>
									<span class="progress-text">
										<?php
										if ( $positive || $negative ) {
											echo "<progress id='file' value='$positive' max='$sum_votes' title='$positive_title$negative_title'> </progress>";
										} else {
											esc_html_e('No rates', 'eazydocs-pro' );
										}
										?>
									</span>
								</div>
							</div>
						</div>
					</li>
					<?php
					}
				}
			}

			wp_reset_postdata();

		} else {
			// Return an error message if the total_page parameter is not set
			echo 'Error: total_page parameter is missing';
		}
	} else {
		if (isset($_GET['total_page'])) {
			$total_page = intval($_GET['total_page']);

			$posts     = get_posts( [ 'post_type' => 'docs', 'posts_per_page'   => -1, 'inclusive' => true] );

			$post_data = [];
			foreach ( $posts as $key => $post ) {
				$post_data[$key]['post_id'] = $post->ID;
				$post_data[$key]['post_title'] = $post->post_title;
				$post_data[$key]['post_edit_link'] = get_edit_post_link( $post->ID );
				$post_data[$key]['post_permalink'] = get_permalink( $post->ID );
				// sum of total positive votes for a post
				$post_data[$key]['positive_time'] = array_sum( get_post_meta( $post->ID, 'positive', false ) );

				$post_data[$key]['negative_time'] = array_sum( get_post_meta( $post->ID, 'negative', false ) );
			}
			// if post has positive_time number large then negative_time number then show large number first
			usort( $post_data, function( $a, $b ) {
				return $b['negative_time'] <=> $a['negative_time'];
			} );

			foreach ( $post_data as $key => $post ) {
				 if ( $post['negative_time'] > 0 ) {
					if ( $key >= 0 && $key <= $total_page ) {
						?>
						<li class="dd-item dd3-item dd-item-parent easydocs-accordion-item  ez-section-acc-item type-docs" data-id="<?php echo $post['post_id']; ?>">
							<div class="dd3-content">
								<div class="accordion-title ez-section-title expand--child has-child">
									<div class="left-content">
										<?php
										$edit_link = 'javascript:void(0)';
										$target = '_self';
										if (current_user_can('editor') || current_user_can('administrator')) {
											$edit_link = $post['post_edit_link'];
											$target = '_blank';
										}
										?>
										<h4>
											<a href="<?php echo esc_attr($edit_link); ?>" target="<?php echo esc_attr($target); ?>">
												<?php echo $post['post_title']; ?>
											</a>
										</h4>
										<ul class="actions">
											<li>
												<a href="<?php echo $post['post_permalink']; ?>" target="_blank" title="<?php esc_attr_e('View this doc item in new tab', 'eazydocs-pro') ?>">
													<span class="dashicons dashicons-external"></span>
												</a>
											</li>
										</ul>
									</div>
									<div class="right-content">
										<?php
										$positive = $post['positive_time'];
										$negative = $post['negative_time'];

										$positive_title      = $positive ? sprintf(_n('%d Positive vote, ', '%d Positive votes and ', $positive, 'eazydocs-pro' ), number_format_i18n($positive)) : esc_html__('No Positive votes, ', 'eazydocs-pro' );
										$negative_title      = $negative ? sprintf(_n('%d Negative vote found.', '%d Negative votes found.', $negative, 'eazydocs-pro' ), number_format_i18n($negative)) : esc_html__('No Negative votes.', 'eazydocs-pro' );

										$positive_icon = '';
										if ( $positive | $negative ) {
											$positive_icon .= '<div class="votes">';
											$positive_icon .= $positive > 0 ? '<div class="like"> <span class="t-success dashicons dashicons-thumbs-up"></span>'.$positive.'</div>' : '';
											$positive_icon .= '<div class="dislike"> <span class="t-danger dashicons dashicons-thumbs-down"></span> '.$negative .'</div>';
											$positive_icon .= '</div>';
										}

										$sum_votes = $positive + $negative;
										echo $positive_icon;
										?>
										<span class="progress-text">
									<?php
									if ( $positive || $negative ) {
										echo "<progress id='file' value='$positive' max='$sum_votes' title='$positive_title$negative_title'> </progress>";
									} else {
										esc_html_e('No rates', 'eazydocs-pro' );
									}
									?>
									</span>
									</div>
								</div>
							</div>
						</li>
						<?php
					}
	
				}
			}
		} else {
			// Return an error message if the total_page parameter is not set
			echo 'Error: total_page parameter is missing';
		}
	}
    wp_die(); // Always remember to call wp_die() after sending an AJAX response
}

// Register image size for embed post
add_image_size( 'ezd_embed_thumb', 100, 100, true );